@@ -1,4 +1,7 @@
 # flake8: noqa
+import logging
+
+import av.logging
 
 from .about import __version__
 from .exceptions import InvalidAccessError, InvalidStateError
@@ -44,3 +47,9 @@ from .stats import (
     RTCStatsReport,
     RTCTransportStats,
 )
+
+# Disable PyAV's logging framework as it can lead to thread deadlocks.
+av.logging.restore_default_callback()
+
+# Set default logging handler to avoid "No handler found" warnings.
+logging.getLogger(__name__).addHandler(logging.NullHandler())
