@@ -1,15 +1,21 @@
 import asyncio
 import copy
+import logging
 import uuid
 from collections import OrderedDict
 from typing import Dict, List, Optional, Set, Union
 
-from pyee import AsyncIOEventEmitter
+from pyee.asyncio import AsyncIOEventEmitter
 
 from . import clock, rtp, sdp
 from .codecs import CODECS, HEADER_EXTENSIONS, is_rtx
 from .events import RTCTrackEvent
-from .exceptions import InternalError, InvalidAccessError, InvalidStateError
+from .exceptions import (
+    InternalError,
+    InvalidAccessError,
+    InvalidStateError,
+    OperationError,
+)
 from .mediastreams import MediaStreamTrack
 from .rtcconfiguration import RTCConfiguration
 from .rtcdatachannel import RTCDataChannel, RTCDataChannelParameters
@@ -41,6 +47,8 @@ DISCARD_HOST = "0.0.0.0"
 DISCARD_PORT = 9
 MEDIA_KINDS = ["audio", "video"]
 
+logger = logging.getLogger(__name__)
+
 
 def filter_preferred_codecs(
     codecs: List[RTCRtpCodecParameters], preferred: List[RTCRtpCodecCapability]
@@ -149,13 +157,14 @@ def add_transport_description(
         media.dtls.fingerprints = dtlsTransport.getLocalParameters().fingerprints
 
 
-def add_remote_candidates(
+async def add_remote_candidates(
     iceTransport: RTCIceTransport, media: sdp.MediaDescription
 ) -> None:
-    for candidate in media.ice_candidates:
-        iceTransport.addRemoteCandidate(candidate)
+    coros = map(iceTransport.addRemoteCandidate, media.ice_candidates)
+    await asyncio.gather(*coros)
+
     if media.ice_candidates_complete:
-        iceTransport.addRemoteCandidate(None)
+        await iceTransport.addRemoteCandidate(None)
 
 
 def allocate_mid(mids: Set[str]) -> str:
@@ -273,13 +282,14 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         self.__certificates = [RTCCertificate.generateCertificate()]
         self.__cname = f"{uuid.uuid4()}"
         self.__configuration = configuration or RTCConfiguration()
+        self.__dtlsTransports: Set[RTCDtlsTransport] = set()
         self.__iceTransports: Set[RTCIceTransport] = set()
         self.__remoteDtls: Dict[
             Union[RTCRtpTransceiver, RTCSctpTransport], RTCDtlsParameters
-        ] = ({})
+        ] = {}
         self.__remoteIce: Dict[
             Union[RTCRtpTransceiver, RTCSctpTransport], RTCIceParameters
-        ] = ({})
+        ] = {}
         self.__seenMids: Set[str] = set()
         self.__sctp: Optional[RTCSctpTransport] = None
         self.__sctp_mline_index: Optional[int] = None
@@ -289,6 +299,7 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         self.__stream_id = str(uuid.uuid4())
         self.__transceivers: List[RTCRtpTransceiver] = []
 
+        self.__connectionState = "new"
         self.__iceConnectionState = "new"
         self.__iceGatheringState = "new"
         self.__isClosed = False
@@ -299,12 +310,37 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         self.__pendingLocalDescription: Optional[sdp.SessionDescription] = None
         self.__pendingRemoteDescription: Optional[sdp.SessionDescription] = None
 
+    @property
+    def connectionState(self) -> str:
+        """
+        The current connection state.
+
+        Possible values: `"connected"`, `"connecting"`, `"closed"`, `"failed"`, `"new`".
+
+        When the state changes, the `"connectionstatechange"` event is fired.
+        """
+        return self.__connectionState
+
     @property
     def iceConnectionState(self) -> str:
+        """
+        The current ICE connection state.
+
+        Possible values: `"checking"`, `"completed"`, `"closed"`, `"failed"`, `"new`".
+
+        When the state changes, the `"iceconnectionstatechange"` event is fired.
+        """
         return self.__iceConnectionState
 
     @property
     def iceGatheringState(self) -> str:
+        """
+        The current ICE gathering state.
+
+        Possible values: `"complete"`, `"gathering"`, `"new`".
+
+        When the state changes, the `"icegatheringstatechange"` event is fired.
+        """
         return self.__iceGatheringState
 
     @property
@@ -333,9 +369,16 @@ class RTCPeerConnection(AsyncIOEventEmitter):
 
     @property
     def signalingState(self):
+        """
+        The current signaling state.
+
+        Possible values: `"closed"`, `"have-local-offer"`, `"have-remote-offer`", `"stable"`.
+
+        When the state changes, the `"signalingstatechange"` event is fired.
+        """
         return self.__signalingState
 
-    def addIceCandidate(self, candidate: RTCIceCandidate) -> None:
+    async def addIceCandidate(self, candidate: RTCIceCandidate) -> None:
         """
         Add a new :class:`RTCIceCandidate` received from the remote peer.
 
@@ -349,7 +392,7 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         for transceiver in self.__transceivers:
             if candidate.sdpMid == transceiver.mid and not transceiver._bundled:
                 iceTransport = transceiver._transport.transport
-                iceTransport.addRemoteCandidate(candidate)
+                await iceTransport.addRemoteCandidate(candidate)
                 return
 
         if (
@@ -358,7 +401,7 @@ class RTCPeerConnection(AsyncIOEventEmitter):
             and not self.__sctp._bundled
         ):
             iceTransport = self.__sctp.transport.transport
-            iceTransport.addRemoteCandidate(candidate)
+            await iceTransport.addRemoteCandidate(candidate)
 
     def addTrack(self, track: MediaStreamTrack) -> RTCRtpSender:
         """
@@ -439,7 +482,11 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         if self.__sctp:
             await self.__sctp.transport.stop()
             await self.__sctp.transport.transport.stop()
+
+        # update states
+        self.__updateIceGatheringState()
         self.__updateIceConnectionState()
+        self.__updateConnectionState()
 
         # no more events will be emitted, so remove all event listeners
         # to facilitate garbage collection.
@@ -754,6 +801,7 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         self.__validate_description(description, is_local=False)
 
         # apply description
+        iceCandidates: Dict[RTCIceTransport, sdp.MediaDescription] = {}
         trackEvents = []
         for i, media in enumerate(description.media):
             dtlsTransport: Optional[RTCDtlsTransport] = None
@@ -777,7 +825,14 @@ class RTCPeerConnection(AsyncIOEventEmitter):
                     find_common_codecs(CODECS[media.kind], media.rtp.codecs),
                     transceiver._preferred_codecs,
                 )
-                assert len(common)
+
+                if not len(common):
+                    raise OperationError(
+                        "Failed to set remote {} description send parameters".format(
+                            media.kind
+                        )
+                    )
+
                 transceiver._codecs = common
                 transceiver._headerExtensions = find_common_header_extensions(
                     HEADER_EXTENSIONS[media.kind], media.rtp.headerExtensions
@@ -835,7 +890,7 @@ class RTCPeerConnection(AsyncIOEventEmitter):
             if dtlsTransport is not None:
                 # add ICE candidates
                 iceTransport = dtlsTransport.transport
-                add_remote_candidates(iceTransport, media)
+                iceCandidates[iceTransport] = media
 
                 # set ICE role
                 if description.type == "offer" and not iceTransport._role_set:
@@ -884,11 +939,21 @@ class RTCPeerConnection(AsyncIOEventEmitter):
             for dtlsTransport in oldTransports:
                 await dtlsTransport.stop()
                 await dtlsTransport.transport.stop()
+                self.__dtlsTransports.discard(dtlsTransport)
                 self.__iceTransports.discard(dtlsTransport.transport)
+                iceCandidates.pop(dtlsTransport.transport, None)
             self.__updateIceGatheringState()
             self.__updateIceConnectionState()
+            self.__updateConnectionState()
 
-        # FIXME: in aiortc 1.0.0 emit RTCTrackEvent directly
+        # add remote candidates
+        coros = [
+            add_remote_candidates(iceTransport, media)
+            for iceTransport, media in iceCandidates.items()
+        ]
+        await asyncio.gather(*coros)
+
+        # FIXME: in aiortc 2.0.0 emit RTCTrackEvent directly
         for event in trackEvents:
             self.emit("track", event.track)
 
@@ -960,13 +1025,20 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         iceGatherer.on("statechange", self.__updateIceGatheringState)
         iceTransport = RTCIceTransport(iceGatherer)
         iceTransport.on("statechange", self.__updateIceConnectionState)
+        iceTransport.on("statechange", self.__updateConnectionState)
         self.__iceTransports.add(iceTransport)
 
+        # create DTLS transport
+        dtlsTransport = RTCDtlsTransport(iceTransport, self.__certificates)
+        dtlsTransport.on("statechange", self.__updateConnectionState)
+        self.__dtlsTransports.add(dtlsTransport)
+
         # update states
         self.__updateIceGatheringState()
         self.__updateIceConnectionState()
+        self.__updateConnectionState()
 
-        return RTCDtlsTransport(iceTransport, self.__certificates)
+        return dtlsTransport
 
     def __createSctpTransport(self) -> None:
         self.__sctp = RTCSctpTransport(self.__createDtlsTransport())
@@ -1016,6 +1088,9 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         rtp.rtcp.mux = True
         return rtp
 
+    def __log_debug(self, msg: str, *args) -> None:
+        logger.debug(f"RTCPeerConnection() {msg}", *args)
+
     def __remoteDescription(self) -> Optional[sdp.SessionDescription]:
         return self.__pendingRemoteDescription or self.__currentRemoteDescription
 
@@ -1029,7 +1104,7 @@ class RTCPeerConnection(AsyncIOEventEmitter):
             rtcp=media.rtp.rtcp,
         )
         if len(media.ssrc):
-            encodings: OrderedDict[int, RTCRtpDecodingParameters] = (OrderedDict())
+            encodings: OrderedDict[int, RTCRtpDecodingParameters] = OrderedDict()
             for codec in transceiver._codecs:
                 if is_rtx(codec):
                     if codec.parameters["apt"] in encodings and len(media.ssrc) == 2:
@@ -1048,8 +1123,37 @@ class RTCPeerConnection(AsyncIOEventEmitter):
         self.__signalingState = state
         self.emit("signalingstatechange")
 
+    def __updateConnectionState(self) -> None:
+        # compute new state
+        # NOTE: we do not have a "disconnected" state
+        dtlsStates = set(map(lambda x: x.state, self.__dtlsTransports))
+        iceStates = set(map(lambda x: x.state, self.__iceTransports))
+        if self.__isClosed:
+            state = "closed"
+        elif "failed" in iceStates or "failed" in dtlsStates:
+            state = "failed"
+        elif not iceStates.difference(["new", "closed"]) and not dtlsStates.difference(
+            ["new", "closed"]
+        ):
+            state = "new"
+        elif "checking" in iceStates or "connecting" in dtlsStates:
+            state = "connecting"
+        elif "new" in dtlsStates:
+            # this avoids a spurious connecting -> connected -> connecting
+            # transition after ICE connects but before DTLS starts
+            state = "connecting"
+        else:
+            state = "connected"
+
+        # update state
+        if state != self.__connectionState:
+            self.__log_debug("connectionState %s -> %s", self.__connectionState, state)
+            self.__connectionState = state
+            self.emit("connectionstatechange")
+
     def __updateIceConnectionState(self) -> None:
         # compute new state
+        # NOTE: we do not have "connected" or "disconnected" states
         states = set(map(lambda x: x.state, self.__iceTransports))
         if self.__isClosed:
             state = "closed"
@@ -1064,6 +1168,9 @@ class RTCPeerConnection(AsyncIOEventEmitter):
 
         # update state
         if state != self.__iceConnectionState:
+            self.__log_debug(
+                "iceConnectionState %s -> %s", self.__iceConnectionState, state
+            )
             self.__iceConnectionState = state
             self.emit("iceconnectionstatechange")
 
@@ -1079,6 +1186,9 @@ class RTCPeerConnection(AsyncIOEventEmitter):
 
         # update state
         if state != self.__iceGatheringState:
+            self.__log_debug(
+                "iceGatheringState %s -> %s", self.__iceGatheringState, state
+            )
             self.__iceGatheringState = state
             self.emit("icegatheringstatechange")
 
