@@ -45,7 +45,7 @@ from .stats import (
 )
 from .utils import uint16_add, uint16_gt
 
-logger = logging.getLogger("rtp")
+logger = logging.getLogger(__name__)
 
 
 def decoder_worker(loop, input_q, output_q):
@@ -254,11 +254,12 @@ class RTCRtpReceiver:
             self.__nack_generator = None
             self.__remote_bitrate_estimator = None
         else:
-            self.__jitter_buffer = JitterBuffer(capacity=128)
+            self.__jitter_buffer = JitterBuffer(capacity=128, is_video=True)
             self.__nack_generator = NackGenerator()
             self.__remote_bitrate_estimator = RemoteBitrateEstimator()
         self._track: Optional[RemoteStreamTrack] = None
         self.__rtcp_exited = asyncio.Event()
+        self.__rtcp_started = asyncio.Event()
         self.__rtcp_task: Optional[asyncio.Future[None]] = None
         self.__rtx_ssrc: Dict[int, int] = {}
         self.__started = False
@@ -378,6 +379,9 @@ class RTCRtpReceiver:
         if self.__started:
             self.__transport._unregister_rtp_receiver(self)
             self.__stop_decoder()
+
+            # shutdown RTCP task
+            await self.__rtcp_started.wait()
             self.__rtcp_task.cancel()
             await self.__rtcp_exited.wait()
 
@@ -487,7 +491,10 @@ class RTCRtpReceiver:
             return
 
         # try to re-assemble encoded frame
-        encoded_frame = self.__jitter_buffer.add(packet)
+        pli_flag, encoded_frame = self.__jitter_buffer.add(packet)
+        # check if the PLI should be sent
+        if pli_flag:
+            await self._send_rtcp_pli(packet.ssrc)
 
         # if we have a complete encoded frame, decode it
         if encoded_frame is not None and self.__decoder_thread:
@@ -498,6 +505,7 @@ class RTCRtpReceiver:
 
     async def _run_rtcp(self) -> None:
         self.__log_debug("- RTCP started")
+        self.__rtcp_started.set()
 
         try:
             while True:
@@ -579,4 +587,4 @@ class RTCRtpReceiver:
             self.__decoder_thread = None
 
     def __log_debug(self, msg: str, *args) -> None:
-        logger.debug(f"receiver(%s) {msg}", self.__kind, *args)
+        logger.debug(f"RTCRtpReceiver(%s) {msg}", self.__kind, *args)
