@@ -22,21 +22,15 @@ from typing import (
     no_type_check,
 )
 
-from pyee import AsyncIOEventEmitter
+from google_crc32c import value as crc32c
+from pyee.asyncio import AsyncIOEventEmitter
 
 from .exceptions import InvalidStateError
 from .rtcdatachannel import RTCDataChannel, RTCDataChannelParameters
 from .rtcdtlstransport import RTCDtlsTransport
 from .utils import random32, uint16_add, uint16_gt, uint32_gt, uint32_gte
 
-try:
-    from crc32c import crc32
-except ImportError:  # pragma: no cover
-    os.environ["CRC32C_SW_MODE"] = "1"
-    from crc32c import crc32
-
-
-logger = logging.getLogger("sctp")
+logger = logging.getLogger(__name__)
 
 # local constants
 COOKIE_LENGTH = 24
@@ -60,7 +54,7 @@ SCTP_RTO_BETA = 1 / 4
 SCTP_RTO_INITIAL = 3.0
 SCTP_RTO_MIN = 1
 SCTP_RTO_MAX = 60
-SCTP_TSN_MODULO = 2 ** 32
+SCTP_TSN_MODULO = 2**32
 
 RECONFIG_MAX_STREAMS = 135
 
@@ -115,8 +109,8 @@ def encode_params(params: List[Tuple[int, bytes]]) -> bytes:
     return body
 
 
-def padl(l: int) -> int:
-    m = l % 4
+def padl(length: int) -> int:
+    m = length % 4
     return 4 - m if m else 0
 
 
@@ -406,7 +400,7 @@ def parse_packet(data: bytes) -> Tuple[int, int, int, List[Any]]:
 
     # verify checksum
     checksum = unpack_from("<L", data, 8)[0]
-    if checksum != crc32(data[0:8] + b"\x00\x00\x00\x00" + data[12:]):
+    if checksum != crc32c(data[0:8] + b"\x00\x00\x00\x00" + data[12:]):
         raise ValueError("SCTP packet has invalid checksum")
 
     chunks = []
@@ -426,7 +420,7 @@ def serialize_packet(
 ) -> bytes:
     header = pack("!HHL", source_port, destination_port, verification_tag)
     data = bytes(chunk)
-    checksum = crc32(header + b"\x00\x00\x00\x00" + data)
+    checksum = crc32c(header + b"\x00\x00\x00\x00" + data)
     return header + pack("<L", checksum) + data
 
 
@@ -732,7 +726,9 @@ class RTCSctpTransport(AsyncIOEventEmitter):
 
             # configure logging
             if logger.isEnabledFor(logging.DEBUG):
-                prefix = self.is_server and "server " or "client "
+                prefix = "RTCSctpTransport(%s) " % (
+                    self.is_server and "server" or "client"
+                )
                 self.__log_debug = lambda msg, *args: logger.debug(prefix + msg, *args)
 
             # initialise local channel ID counter
