@@ -1,8 +1,12 @@
+import math
 import os
+import struct
 from dataclasses import dataclass, field
 from struct import pack, unpack, unpack_from
 from typing import Any, List, Optional, Tuple, Union
 
+from av import AudioFrame
+
 from .rtcrtpparameters import RTCRtpParameters
 
 # reserved to avoid confusion with RTCP
@@ -209,11 +213,11 @@ def is_rtcp(msg: bytes) -> bool:
     return len(msg) >= 2 and msg[1] >= 192 and msg[1] <= 208
 
 
-def padl(l: int) -> int:
+def padl(length: int) -> int:
     """
     Return amount of padding needed for a 4-byte multiple.
     """
-    return 4 * ((l + 3) // 4) - l
+    return 4 * ((length + 3) // 4) - length
 
 
 def unpack_header_extensions(
@@ -303,6 +307,29 @@ def pack_header_extensions(extensions: List[Tuple[int, bytes]]) -> Tuple[int, by
     return extension_profile, extension_value
 
 
+def compute_audio_level_dbov(frame: AudioFrame):
+    """
+    Compute the energy level as spelled out in RFC 6465, Appendix A.
+    """
+    MAX_SAMPLE_VALUE = 32767
+    MAX_AUDIO_LEVEL = 0
+    MIN_AUDIO_LEVEL = -127
+    rms = 0
+    buf = bytes(frame.planes[0])
+    s = struct.Struct("h")
+    for unpacked in s.iter_unpack(buf):
+        sample = unpacked[0]
+        rms += sample * sample
+    rms = math.sqrt(rms / (frame.samples * MAX_SAMPLE_VALUE * MAX_SAMPLE_VALUE))
+    if rms > 0:
+        db = 20 * math.log10(rms)
+        db = max(db, MIN_AUDIO_LEVEL)
+        db = min(db, MAX_AUDIO_LEVEL)
+    else:
+        db = MIN_AUDIO_LEVEL
+    return round(db)
+
+
 @dataclass
 class RtcpReceiverInfo:
     ssrc: int
@@ -389,7 +416,7 @@ class RtcpByePacket:
 
 @dataclass
 class RtcpPsfbPacket:
-    """"
+    """
     Payload-Specific Feedback Message (RFC 4585).
     """
 
