@@ -31,7 +31,7 @@ from aiortc.rtp import (
     pack_remb_fci,
 )
 
-from .utils import dummy_ice_transport_pair, load, run
+from .utils import asynctest, dummy_ice_transport_pair, load
 
 RTP = load("rtp.bin")
 RTCP = load("rtcp_sr.bin")
@@ -102,7 +102,8 @@ class RTCDtlsTransportTest(TestCase):
         self.assertEqual(stats_b.bytesSent, stats_a.bytesReceived)
 
     @patch("aiortc.rtcdtlstransport.lib.SSL_CTX_use_certificate")
-    def test_broken_ssl(self, mock_use_certificate):
+    @asynctest
+    async def test_broken_ssl(self, mock_use_certificate):
         mock_use_certificate.return_value = 0
 
         transport1, transport2 = dummy_ice_transport_pair()
@@ -111,7 +112,8 @@ class RTCDtlsTransportTest(TestCase):
         with self.assertRaises(DtlsError):
             RTCDtlsTransport(transport1, [certificate])
 
-    def test_data(self):
+    @asynctest
+    async def test_data(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
@@ -124,37 +126,36 @@ class RTCDtlsTransportTest(TestCase):
         receiver2 = DummyDataReceiver()
         session2._register_data_receiver(receiver2)
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
 
         # send encypted data
-        run(session1._send_data(b"ping"))
-        run(asyncio.sleep(0.1))
+        await session1._send_data(b"ping")
+        await asyncio.sleep(0.1)
         self.assertEqual(receiver2.data, [b"ping"])
 
-        run(session2._send_data(b"pong"))
-        run(asyncio.sleep(0.1))
+        await session2._send_data(b"pong")
+        await asyncio.sleep(0.1)
         self.assertEqual(receiver1.data, [b"pong"])
 
         # shutdown
-        run(session1.stop())
-        run(asyncio.sleep(0.1))
+        await session1.stop()
+        await asyncio.sleep(0.1)
         self.assertEqual(session1.state, "closed")
         self.assertEqual(session2.state, "closed")
 
         # try closing again
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
         # try sending after close
         with self.assertRaises(ConnectionError):
-            run(session1._send_data(b"foo"))
+            await session1._send_data(b"foo")
 
-    def test_data_handler_error(self):
+    @asynctest
+    async def test_data_handler_error(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
@@ -166,22 +167,21 @@ class RTCDtlsTransportTest(TestCase):
         session2 = RTCDtlsTransport(transport2, [certificate2])
         session2._register_data_receiver(BrokenDataReceiver())
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
 
         # send encypted data
-        run(session1._send_data(b"ping"))
-        run(asyncio.sleep(0.1))
+        await session1._send_data(b"ping")
+        await asyncio.sleep(0.1)
 
         # shutdown
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
-    def test_rtp(self):
+    @asynctest
+    async def test_rtp(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
@@ -214,56 +214,56 @@ class RTCDtlsTransportTest(TestCase):
             ),
         )
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
         self.assertCounters(session1, session2, 2, 2)
 
         # send RTP
-        run(session1._send_rtp(RTP))
-        run(asyncio.sleep(0.1))
+        await session1._send_rtp(RTP)
+        await asyncio.sleep(0.1)
         self.assertCounters(session1, session2, 3, 2)
         self.assertEqual(len(receiver2.rtcp_packets), 0)
         self.assertEqual(len(receiver2.rtp_packets), 1)
 
         # send RTCP
-        run(session2._send_rtp(RTCP))
-        run(asyncio.sleep(0.1))
+        await session2._send_rtp(RTCP)
+        await asyncio.sleep(0.1)
         self.assertCounters(session1, session2, 3, 3)
         self.assertEqual(len(receiver1.rtcp_packets), 1)
         self.assertEqual(len(receiver1.rtp_packets), 0)
 
         # shutdown
-        run(session1.stop())
-        run(asyncio.sleep(0.1))
+        await session1.stop()
+        await asyncio.sleep(0.1)
         self.assertCounters(session1, session2, 4, 3)
         self.assertEqual(session1.state, "closed")
         self.assertEqual(session2.state, "closed")
 
         # try closing again
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
         # try sending after close
         with self.assertRaises(ConnectionError):
-            run(session1._send_rtp(RTP))
+            await session1._send_rtp(RTP)
 
-    def test_rtp_malformed(self):
+    @asynctest
+    async def test_rtp_malformed(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
         session1 = RTCDtlsTransport(transport1, [certificate1])
 
         # receive truncated RTP
-        run(session1._handle_rtp_data(RTP[0:8], 0))
+        await session1._handle_rtp_data(RTP[0:8], 0)
 
         # receive truncated RTCP
-        run(session1._handle_rtcp_data(RTCP[0:8]))
+        await session1._handle_rtcp_data(RTCP[0:8])
 
-    def test_srtp_unprotect_error(self):
+    @asynctest
+    async def test_srtp_unprotect_error(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
@@ -296,26 +296,25 @@ class RTCDtlsTransportTest(TestCase):
             ),
         )
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
 
         # send same RTP twice, to trigger error on the receiver side:
         # "replay check failed (bad index)"
-        run(session1._send_rtp(RTP))
-        run(session1._send_rtp(RTP))
-        run(asyncio.sleep(0.1))
+        await session1._send_rtp(RTP)
+        await session1._send_rtp(RTP)
+        await asyncio.sleep(0.1)
         self.assertEqual(len(receiver2.rtcp_packets), 0)
         self.assertEqual(len(receiver2.rtp_packets), 1)
 
         # shutdown
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
-    def test_abrupt_disconnect(self):
+    @asynctest
+    async def test_abrupt_disconnect(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
@@ -324,26 +323,26 @@ class RTCDtlsTransportTest(TestCase):
         certificate2 = RTCCertificate.generateCertificate()
         session2 = RTCDtlsTransport(transport2, [certificate2])
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
 
         # break connections -> tasks exits
-        run(transport1.stop())
-        run(transport2.stop())
+        await transport1.stop()
+        await transport2.stop()
+        await asyncio.sleep(0.1)
 
         # close DTLS
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
         # check outcome
         self.assertEqual(session1.state, "closed")
         self.assertEqual(session2.state, "closed")
 
-    def test_abrupt_disconnect_2(self):
+    @asynctest
+    async def test_abrupt_disconnect_2(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
@@ -352,11 +351,9 @@ class RTCDtlsTransportTest(TestCase):
         certificate2 = RTCCertificate.generateCertificate()
         session2 = RTCDtlsTransport(transport2, [certificate2])
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
 
         def fake_write_ssl():
@@ -365,14 +362,16 @@ class RTCDtlsTransportTest(TestCase):
         session1._write_ssl = fake_write_ssl
 
         # close DTLS -> ConnectionError
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
+        await asyncio.sleep(0.1)
 
         # check outcome
         self.assertEqual(session1.state, "closed")
         self.assertEqual(session2.state, "closed")
 
-    def test_bad_client_fingerprint(self):
+    @asynctest
+    async def test_bad_client_fingerprint(self):
         transport1, transport2 = dummy_ice_transport_pair()
 
         certificate1 = RTCCertificate.generateCertificate()
@@ -386,22 +385,21 @@ class RTCDtlsTransportTest(TestCase):
                 RTCDtlsFingerprint(algorithm="sha-256", value="bogus_fingerprint")
             ]
         )
-        run(
-            asyncio.gather(
-                session1.start(bogus_parameters),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(bogus_parameters),
+            session2.start(session1.getLocalParameters()),
         )
         self.assertEqual(session1.state, "failed")
         self.assertEqual(session2.state, "connected")
 
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
     @patch("aiortc.rtcdtlstransport.lib.SSL_do_handshake")
     @patch("aiortc.rtcdtlstransport.lib.SSL_get_error")
     @patch("aiortc.rtcdtlstransport.lib.ERR_get_error")
-    def test_handshake_error(
+    @asynctest
+    async def test_handshake_error(
         self, mock_err_get_error, mock_ssl_get_error, mock_do_handshake
     ):
         mock_err_get_error.side_effect = [0x2006D080, 0, 0]
@@ -416,19 +414,18 @@ class RTCDtlsTransportTest(TestCase):
         certificate2 = RTCCertificate.generateCertificate()
         session2 = RTCDtlsTransport(transport2, [certificate2])
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
         self.assertEqual(session1.state, "failed")
         self.assertEqual(session2.state, "failed")
 
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
-    def test_lossy_channel(self):
+    @asynctest
+    async def test_lossy_channel(self):
         """
         Transport with 25% loss eventually connects.
         """
@@ -443,15 +440,13 @@ class RTCDtlsTransportTest(TestCase):
         certificate2 = RTCCertificate.generateCertificate()
         session2 = RTCDtlsTransport(transport2, [certificate2])
 
-        run(
-            asyncio.gather(
-                session1.start(session2.getLocalParameters()),
-                session2.start(session1.getLocalParameters()),
-            )
+        await asyncio.gather(
+            session1.start(session2.getLocalParameters()),
+            session2.start(session1.getLocalParameters()),
         )
 
-        run(session1.stop())
-        run(session2.stop())
+        await session1.stop()
+        await session2.stop()
 
 
 class RtpRouterTest(TestCase):
@@ -555,3 +550,29 @@ class RtpRouterTest(TestCase):
 
         # unknown SSRC and payload type
         self.assertEqual(router.route_rtp(RtpPacket(ssrc=6789, payload_type=100)), None)
+
+    def test_route_rtp_ambiguous_payload_type(self):
+        receiver1 = object()
+        receiver2 = object()
+
+        router = RtpRouter()
+        router.register_receiver(receiver1, ssrcs=[1234, 2345], payload_types=[96, 97])
+        router.register_receiver(receiver2, ssrcs=[3456, 4567], payload_types=[96, 97])
+
+        # known SSRC and payload type
+        self.assertEqual(
+            router.route_rtp(RtpPacket(ssrc=1234, payload_type=96)), receiver1
+        )
+        self.assertEqual(
+            router.route_rtp(RtpPacket(ssrc=2345, payload_type=97)), receiver1
+        )
+        self.assertEqual(
+            router.route_rtp(RtpPacket(ssrc=3456, payload_type=96)), receiver2
+        )
+        self.assertEqual(
+            router.route_rtp(RtpPacket(ssrc=4567, payload_type=97)), receiver2
+        )
+
+        # unknown SSRC, ambiguous payload type
+        self.assertEqual(router.route_rtp(RtpPacket(ssrc=5678, payload_type=96)), None)
+        self.assertEqual(router.route_rtp(RtpPacket(ssrc=5678, payload_type=97)), None)
